import re
import os

import pyglet

import scenes
from colors import *

# Resources
floor = pyglet.resource.image('floor.png')
wall = pyglet.resource.image('wall.png')
human = pyglet.resource.image('human.png')
robot = pyglet.resource.image('robot.png')
turret = pyglet.resource.image('turret.png')
trap = pyglet.resource.image('trap.png')
exit = pyglet.resource.image('exit.png')
fuel = pyglet.resource.image('fuel.png')
background = pyglet.resource.image('background.png')
pat = pyglet.image.SolidColorImagePattern((0, 0, 0, 30))
frame = pat.create_image(150, 150)
p_frame = pat.create_image(418, 50)
p_front = pyglet.image.SolidColorImagePattern((40, 45, 47, 255)).create_image(418, 50)


class Prompt(object):
    def __init__(self):
        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.doc = pyglet.text.document.UnformattedDocument()
        self.doc.set_style(0, 0, {'color': (171, 215, 95, 155), 'font_size': 18})
        self.layout = pyglet.text.layout.IncrementalTextLayout(self.doc, 400, 40)
        self.layout.x = 100
        self.layout.y = 300
        self.front = pyglet.sprite.Sprite(p_front, self.layout.x-10, self.layout.y-15)
        self.frame = pyglet.sprite.Sprite(p_frame, self.layout.x-20, self.layout.y-5)
        self.caret = pyglet.text.caret.Caret(self.layout)
        self.caret.color = grey[:-1]
        self.text = pyglet.text.Label('Please enter a level name:',
                        x=60, y=400,
                        font_name="Another Typewriter", font_size=16, color=grey)


    def on_text(self, text):
        if ord(text) != 13:
            self.caret.on_text(text)

    def on_text_motion(self, motion):
            self.caret.on_text_motion(motion)

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.set_scene(scenes.Load())
            return pyglet.event.EVENT_HANDLED
        elif symbol == pyglet.window.key.ENTER:
            test = re.compile('\w')
            t = self.doc.text
            l = []
            for char in t:
                if re.match(test, char):
                    l.append(char)
            safe_name = ''.join(l)
            self.window.set_scene(Edit(safe_name))

    def update(self, dt):
        pass

    def draw(self):
        self.background.draw()
        self.frame.draw()
        self.front.draw()
        self.layout.draw()
        self.text.draw()


class Load(object):
    def __init__(self):
        self.batch = pyglet.graphics.Batch()
        self.frame_batch = pyglet.graphics.Batch()
        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.page = 0
        self.thumbs = []
        self.frames = []
        self.maps = os.listdir(os.path.join('data', 'map'))
        self.maps.remove('thumb')
        self.maps.sort()
        self.coords = [[45, 425], [225, 425], [405, 425]
                      ,[45, 245], [225, 245], [405, 245]
                      ,[45, 65], [225, 65], [405, 65]]
        self.next = pyglet.text.Label('Next', x=390, y=23,
                        font_name="Another Typewriter", font_size=16, color=grey)
        self.prev = pyglet.text.Label('Prev', x=140, y=23,
                        font_name="Another Typewriter", font_size=16, color=grey)
        self.new = pyglet.text.Label('New', x=266, y=20,
                        font_name="Another Typewriter", font_size=22, color=(171, 215, 95, 155))
        self.load_page()

    def load_page(self):
        self.thumbs = []
        self.frames = []
        for n, n2 in enumerate(range(self.page*9, self.page*9+9)):
            if n2 < len(self.maps):
                name = self.maps[n2]
                path = os.path.join('data', 'map', 'thumb', name+'.png')
                img = pyglet.image.load(path)
                self.thumbs.append([name, pyglet.sprite.Sprite(img,
                                                           self.coords[n][0],
                                                           self.coords[n][1],
                                                           batch=self.batch)])
                self.frames.append([name, pyglet.sprite.Sprite(frame,
                                                           self.coords[n][0]-10,
                                                           self.coords[n][1]+10,
                                                           batch=self.frame_batch)])
     
    def on_mouse_press(self, x, y, button, modifiers):
        for level, thumb in self.thumbs:
            if x >= thumb.x and x <=  thumb.x + thumb.width and\
                y >= thumb.y and y <= thumb.y + thumb.height:
                    self.window.set_scene(Edit(level))

        if x >= self.new.x and x <=  self.new.x + self.new.content_width and\
            y >= self.new.y and y <= self.new.y + self.new.content_height:
                self.window.set_scene(Prompt())

        if x >= self.next.x and x <=  self.next.x + self.next.content_width and\
            y >= self.next.y and y <= self.next.y + self.next.content_height:
                self.page += 1
                self.load_page()

        if x >= self.prev.x and x <=  self.prev.x + self.prev.content_width and\
            y >= self.prev.y and y <= self.prev.y + self.prev.content_height:
                self.page -= 1
                if self.page < 0:
                    self.window.set_scene(scenes.Menu())
                self.load_page()

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.set_scene(scenes.Menu())
            return pyglet.event.EVENT_HANDLED

    def update(self, dt):
        pass

    def draw(self):
        self.background.draw()
        self.frame_batch.draw()
        self.batch.draw()
        self.next.draw()
        self.prev.draw()
        self.new.draw()


class Cursor(pyglet.window.MouseCursor):
    def __init__(self):
        super(Cursor, self).__init__(0, 0)
        self.sprite = pyglet.sprite.Sprite(wall)
        self.sprite.opacity = 99

    def draw(self, x, y):
        self.sprite.position = (x, y)
        self.sprite.draw()
        

class Edit(object):
    def __init__(self, level):
        self.level = level
        self.batch = pyglet.graphics.Batch()
        self.grid = [[None] * 30 for x in range(30)]
        self.brush = wall
        self.init = False
        self.get_thumb = False
        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.load()
           
    def save(self):
        path = os.path.join('data', 'map', self.level)
        file_ = open(path, 'w')
        for row in range(30):
            for col in range(30):
                sprite = self.grid[row][col]
                if sprite:
                    if sprite.image == wall:
                        file_.write('W')
                    elif sprite.image == human:
                        file_.write('H')
                    elif sprite.image == robot:
                        file_.write('R')
                    elif sprite.image == turret:
                        file_.write('G')
                    elif sprite.image == trap:
                        file_.write('T')
                    elif sprite.image == exit:
                        file_.write('E')
                    elif sprite.image == fuel:
                        file_.write('F')
                else:
                    file_.write('.')
            file_.write('\n')
        file_.close()
        self.get_thumb = True
        

    def load(self):
        path = os.path.join('data', 'map', self.level)
        if not os.access(path, os.F_OK):
            return
        file_ = open(path, 'r')
        lines = file_.readlines()
        file_.close()
        new_lines = []
        for line in lines:
            new_lines.append(list(line.strip()))
        for x in range(len(self.grid)):
            for y in range(len(self.grid[0])):
                char = new_lines[x][y]
                if char == '.':
                    image = None
                elif char == 'W':
                    image = wall
                elif char == 'H':
                    image = human
                elif char == 'R':
                    image = robot
                elif char == 'G':
                    image = turret
                elif char == 'T':
                    image = trap
                elif char == 'E':
                    image = exit
                elif char == 'F':
                    image = fuel

                if image:
                    self.grid[x][y] = pyglet.sprite.Sprite(image, x*20+10, y*20+10,
                                                           batch=self.batch)
                else:
                    self.grid[x][y] = None


    def on_mouse_press(self, x, y, button, modifiers):
        grid_x = 30 - ((600 - x) / 20) - 1
        grid_y = 30 - ((600 - y) / 20) - 1
        if grid_x < 0: grid_x = 0
        if grid_x > 29: grid_x = 29
        if grid_y < 0: grid_y = 0
        if grid_y > 29: grid_y = 29
        if button == pyglet.window.mouse.LEFT:
            self.grid[grid_x][grid_y] = pyglet.sprite.Sprite(self.brush,
                                                             grid_x*20+10,
                                                             grid_y*20+10,
                                                             batch=self.batch)
        elif button == pyglet.window.mouse.RIGHT:
            self.grid[grid_x][grid_y] = None

    def on_mouse_drag(self, x, y, dx, dy, buttons, modifiers):
        grid_x = 30 - ((600 - x) / 20) - 1
        grid_y = 30 - ((600 - y) / 20) - 1
        if grid_x < 0: grid_x = 0
        if grid_x > 29: grid_x = 29
        if grid_y < 0: grid_y = 0
        if grid_y > 29: grid_y = 29
        if buttons & pyglet.window.mouse.LEFT:
            self.grid[grid_x][grid_y] = pyglet.sprite.Sprite(self.brush,
                                                             grid_x*20+10,
                                                             grid_y*20+10,
                                                             batch=self.batch)
        elif buttons & pyglet.window.mouse.RIGHT:
            self.grid[grid_x][grid_y] = None

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.set_scene(scenes.Load())
            self.window.set_mouse_cursor(None)
            return pyglet.event.EVENT_HANDLED
        elif symbol == pyglet.window.key._1:
            self.save()
        elif symbol == pyglet.window.key._2:
            self.window.set_scene(Load())
            self.window.set_mouse_cursor(None)
        elif symbol == pyglet.window.key.A:
            self.brush = wall
        elif symbol == pyglet.window.key.S:
            self.brush = turret
        elif symbol == pyglet.window.key.D:
            self.brush = trap
        elif symbol == pyglet.window.key.W:
            self.brush = human
        elif symbol == pyglet.window.key.E:
            self.brush = robot
        elif symbol == pyglet.window.key.R:
            self.brush = exit
        elif symbol == pyglet.window.key.F:
            self.brush = fuel

    def update(self, dt):
        if not self.init:
            self.window.music.stop()
            self.cursor = Cursor()
            self.window.set_mouse_cursor(self.cursor)
            self.init = True
        self.cursor.sprite.image = self.brush

    def draw(self):
        self.background.draw()
        self.batch.draw()
        if self.get_thumb:
            thumbpath = os.path.join('data', 'map', 'thumb', self.level + '.png')
            img_data = pyglet.image.get_buffer_manager().get_color_buffer().get_image_data()
            img_data.blit(0, 450, 0, 150, 150)
            img_data = pyglet.image.get_buffer_manager().get_color_buffer().get_image_data()
            img_data.width = 150
            img_data.height = 150
            img_data.save(thumbpath)
            self.get_thumb = False
