import os
import math

import pyglet
import pygame

import scenes
from colors import *


# Resources
bullet = pyglet.resource.image('bullet.png')
bullet.anchor_x = bullet.width/2
bullet.anchor_y = bullet.height/2
trap = pyglet.resource.image('trap.png')
trap.anchor_x = trap.width/2
trap.anchor_y = trap.height/2
turret = pyglet.resource.image('turret.png')
turret.anchor_x = turret.width/2
turret.anchor_y = turret.height/2
exit = pyglet.resource.image('exit.png')
exit.anchor_x = exit.width/2
exit.anchor_y = exit.height/2
wall = pyglet.resource.image('wall.png')
wall.anchor_x = wall.width/2
wall.anchor_y = wall.height/2
human = pyglet.resource.image('human.png')
human.anchor_x = human.width/2
human.anchor_y = human.height/2
robot = pyglet.resource.image('robot.png')
robot.anchor_x = robot.width/2
robot.anchor_y = robot.height/2
vocal = pyglet.resource.image('vocal.png')
vocal.anchor_x = vocal.width/2
vocal.anchor_y = vocal.height/2
fuel = pyglet.resource.image('fuel.png')
fuel.anchor_x = fuel.width/2
fuel.anchor_y = fuel.height/2
background = pyglet.resource.image('background.png')

robot_hit = pygame.mixer.Sound(os.path.join('data', 'music', 'robot_hit.ogg'))
bullet_hit = pygame.mixer.Sound(os.path.join('data', 'music', 'bullet_hit.ogg'))
fuel_hit = pygame.mixer.Sound(os.path.join('data', 'music', 'fuel_hit.ogg'))
win_hit = pygame.mixer.Sound(os.path.join('data', 'music', 'win.ogg'))

# Collision
def circle_rectangle(circle_x, circle_y, circle_radius,
                     rect_x, rect_y, rect_width, rect_height):
    rect_x = rect_x - rect_width/2
    rect_y = rect_y - rect_height/2
    testx = circle_x
    testy = circle_y
    if testx < rect_x: testx = rect_x
    if testx > rect_x + rect_width: testx = rect_x + rect_width
    if testy > rect_y + rect_height: testy = rect_y + rect_height
    if testy < rect_y: testy = rect_y
    distance = math.sqrt((testx - circle_x)**2 + (testy - circle_y)**2)
    return distance < circle_radius


# Win Scene
class Win(object):
    def __init__(self):
        win_hit.play()
        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.font = pyglet.font.load("Another Typewriter", 36);
        self.font2 = pyglet.font.load("Another Typewriter", 16);
        self.text = pyglet.font.Text(self.font, 'Level Complete', x=10, y=300,
                                        color=cnormalize((171, 215, 95, 155)))
        self.text2 = pyglet.font.Text(self.font2, 'Press any key...', x=260, y=10,
                                        color=cnormalize(grey));


    def on_key_press(self, symbol, modifiers):
        self.window.set_scene(scenes.Select(Play))
        return pyglet.event.EVENT_HANDLED

    def update(self, dt):
        pass

    def draw(self):
        self.background.draw()
        self.text.draw()
        self.text2.draw()


# Sprites
class Vocal(pyglet.sprite.Sprite):
    def __init__(self, color):
        super(Vocal, self).__init__(vocal, 0, 0)
        self.color = color
        self.opacity = 75

    def update(self, dt):
        pass


class Human(pyglet.sprite.Sprite):
    def __init__(self, x, y):
        super(Human, self).__init__(human, x*20+10, y*20+10)
        self.volume = 31
        self.vocal = Vocal(grey[:-1])
        self.vocal.opacity = 0
        self.dx = 0
        self.dy = 0

    def update(self, dt):
        self.vocal.update(dt)
        if self.vocal.opacity > 0:
            self.vocal.opacity -= 75 * dt
        if self.vocal.opacity < 0:
            self.vocal.opacity = 0
        self.vocal.x = self.x
        self.vocal.y = self.y
        self.x += self.dx * dt
        self.y += self.dy * dt

    def draw(self):
        super(Human, self).draw()
        self.vocal.draw()

    def say(self, word):
        if word == 'go':
            color = green[:-1]
        elif word == 'stop':
            color = red[:-1]
        elif word == 'turn':
            color = dark_yellow[:-1]
        elif word == 'flip':
            color = blue[:-1]
        self.vocal.color = color
        self.vocal.opacity = 75


class Robot(pyglet.sprite.Sprite):
    def __init__(self, x, y):
        super(Robot, self).__init__(robot, x*20+10, y*20+10)
        self.dx = 0
        self.dy = 0
        self.set_direction('N')
        self.moving = False
        self.fuel = 10
        self.font = pyglet.font.load(None, 10)
        self.fuel_label = pyglet.font.Text(self.font, str(self.fuel), color=cnormalize(grey))

    def update(self, dt):
        self.fuel -= 1 * dt
        if self.fuel > 99:
            self.fuel = 99 
        if self.fuel < 0:
            self.fuel = 0
        if self.moving:
            self.x += self.dx * dt
            self.y += self.dy * dt
        self.fuel_label.text = ('0' + str(self.fuel)).split('.')[0][-2:]
        self.fuel_label.x = self.x - 8
        self.fuel_label.y = self.y - 5

    def draw(self):
        super(Robot, self).draw()
        self.fuel_label.draw()

    def add_fuel(self):
        self.fuel += 10

    def set_direction(self, direction):
        self.direction = direction
        speed = 40
        if direction == 'N':
            self.rotation = 0
            self.dx = 0
            self.dy = speed
        elif direction == 'S':
            self.rotation = 180
            self.dx = 0
            self.dy = -speed
        elif direction == 'E':
            self.rotation = 90
            self.dx = speed
            self.dy = 0
        elif direction == 'W':
            self.rotation = 270 
            self.dx = -speed
            self.dy = 0

    def stop(self):
        self.moving = False

    def go(self):
        self.moving = True

    def turn(self):
        if self.direction == 'S':
            self.set_direction('W')
        elif self.direction == 'N':
            self.set_direction('E')
        elif self.direction == 'E':
            self.set_direction('S')
        elif self.direction == 'W':
            self.set_direction('N')

    def flip(self):
        if self.direction == 'S':
            self.set_direction('N')
        elif self.direction == 'N':
            self.set_direction('S')
        elif self.direction == 'E':
            self.set_direction('W')
        elif self.direction == 'W':
            self.set_direction('E')


class Bullet(pyglet.sprite.Sprite):
    def __init__(self, x, y, dx, dy, batch):
        super(Bullet, self).__init__(bullet, x, y, batch=batch)
        self.dx = dx
        self.dy = dy

    def update(self, dt):
        self.x += self.dx * dt
        self.y += self.dy * dt


class Turret(pyglet.sprite.Sprite):
    def __init__(self, x, y, batch):
        super(Turret, self).__init__(turret, x*20+10, y*20+10, batch=batch)
        self.bullets = []
        self.time = 0

    def fire(self):
        speed = 100
        self.bullets.append(Bullet(self.x, self.y, speed, 0, self.batch))
        self.bullets.append(Bullet(self.x, self.y, -speed, 0, self.batch))
        self.bullets.append(Bullet(self.x, self.y, 0, speed, self.batch))
        self.bullets.append(Bullet(self.x, self.y, 0, -speed, self.batch))

    def update(self, dt):
        self.time += dt
        if self.time >= .5:
            self.time = 0
            self.fire()
        for bullet in self.bullets:
            bullet.update(dt)


class Play(pyglet.window.key.KeyStateHandler):
    def __init__(self, level):
        self.level = level

        self.misc_batch = pyglet.graphics.Batch()
        self.wall_batch = pyglet.graphics.Batch()
        self.exit_batch = pyglet.graphics.Batch()

        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.walls = []
        self.traps = []
        self.turrets = []
        self.exits = []
        self.fuel = []
        self.robot = None
        self.human = None

        self.first = True

        self.load_map()

    def load_map(self):
        file_ = open(os.path.join('data', 'map', self.level), 'r')
        lines = file_.readlines()
        file_.close()
        new_lines = []
        for line in lines:
            new_lines.append(list(line.strip()))
        for x in range(30):
            for y in range(30):
                char = new_lines[x][y]
                if char == '.':
                    pass
                elif char == 'W':
                    self.walls.append(pyglet.sprite.Sprite(wall, x*20+10, y*20+10,
                                                           batch=self.wall_batch))
                elif char == 'H':
                    self.human = Human(x, y)
                elif char == 'R':
                    self.robot = Robot(x, y)
                elif char == 'G':
                    self.turrets.append(Turret(x, y, self.misc_batch))
                elif char == 'T':
                    self.traps.append(pyglet.sprite.Sprite(trap, x*20+10, y*20+10,
                                                           batch=self.misc_batch))
                elif char == 'E':
                    self.exits.append(pyglet.sprite.Sprite(exit, x*20+10, y*20+10,
                                                           batch=self.exit_batch))
                elif char == 'F':
                    self.fuel.append(pyglet.sprite.Sprite(fuel, x*20+10, y*20+10,
                                                          batch=self.misc_batch))

    def update(self, dt):
        if self.first:
            self.window.music.stop()
            self.window.set_mouse_visible(False)
            self.first = False
        speed = 100
        if self[pyglet.window.key.LEFT] and self[pyglet.window.key.UP]:
            pass
            #self.human.dx = -speed
            #self.human.dy = speed
            #self.human.rotation = 315
        elif self[pyglet.window.key.LEFT] and self[pyglet.window.key.DOWN]:
            pass
            #self.human.dx = -speed
            #self.human.dy = -speed
            #self.human.rotation = 225
        elif self[pyglet.window.key.RIGHT] and self[pyglet.window.key.UP]:
            pass
            #self.human.dx = speed
            #self.human.dy = speed
            #self.human.rotation = 45
        elif self[pyglet.window.key.RIGHT] and self[pyglet.window.key.DOWN]:
            pass
            #self.human.dx = speed
            #self.human.dy = -speed
            #self.human.rotation = 135
        elif self[pyglet.window.key.LEFT]:
            self.human.dx = -speed
            self.human.dy = 0
            self.human.rotation = 270
        elif self[pyglet.window.key.RIGHT]:
            self.human.dx = speed
            self.human.dy = 0
            self.human.rotation = 90
        elif self[pyglet.window.key.UP]:
            self.human.dx = 0
            self.human.dy = speed
            self.human.rotation = 0
        elif self[pyglet.window.key.DOWN]:
            self.human.dx = 0
            self.human.dy = -speed
            self.human.rotation = 180
        else:
            self.human.dx = 0
            self.human.dy = 0

        self.human.update(dt)
        self.robot.update(dt)
        for turret in self.turrets:
            turret.update(dt)

        # Singles (Human, Robot)
        human_cells = self.col_cells(self.human)
        robot_cells = self.col_cells(self.robot)
        for cell in human_cells:
            if cell in robot_cells:
                if circle_rectangle(self.human.x, self.human.y, self.human.width/2,
                        self.robot.x, self.robot.y, self.robot.width, self.robot.height):
                    self.on_loss()

        grid = {}
        for tile in self.walls + self.traps + self.turrets + self.fuel + self.exits:
            x = (tile.x - 10) / 20
            y = (tile.y - 10) / 20
            grid[(x, y)] = tile

        wall_grid = {}
        for tile in self.walls:
            x = (tile.x - 10) / 20
            y = (tile.y - 10) / 20
            wall_grid[(x, y)] = tile

        human_in_exit = False
        robot_in_exit = False
        for grid_cell in grid.keys():
            if grid_cell in human_cells:
                tile = grid[grid_cell]
                if circle_rectangle(self.human.x, self.human.y, self.human.width/2,
                        tile.x, tile.y, tile.width, tile.height):
                    # Human vs Traps and Turrets
                    if tile in self.traps + self.turrets:
                        self.on_loss()
                    # Human vs Exits
                    elif tile in self.exits:
                        human_in_exit = True
                    # Human vs Walls
                    elif tile in self.walls:
                        if self.human.dx < 0:
                            self.human.x = tile.x + 18
                        elif self.human.dx > 0:
                            self.human.x = tile.x - 18
                        elif self.human.dy < 0:
                            self.human.y = tile.y + 18
                        elif self.human.dy > 0:
                            self.human.y = tile.y - 18
            if grid_cell in robot_cells:
                tile = grid[grid_cell]
                if circle_rectangle(self.robot.x, self.robot.y, self.robot.width/2,
                        tile.x, tile.y, tile.width, tile.height):
                    # Robot vs Exits
                    if tile in self.exits:
                        robot_in_exit = True
                    # Robot vs Walls and Turrets
                    elif tile in self.walls + self.turrets:
                        if self.robot.dx < 0:
                            self.robot.x = tile.x + 20
                        elif self.robot.dx > 0:
                            self.robot.x = tile.x - 20
                        elif self.robot.dy < 0:
                            self.robot.y = tile.y + 20
                        elif self.robot.dy > 0:
                            self.robot.y = tile.y - 20
                        self.robot.moving = False
                        robot_hit.play()
                    # Robot vs Fuel
                    elif tile in self.fuel:
                        fuel_hit.play()
                        self.fuel.remove(tile)
                        self.robot.add_fuel()

        if human_in_exit and robot_in_exit:
            self.on_win()

        if self.robot.fuel <= 0:
            self.on_loss()

        # Bullets vs Human, Robot, and Walls 
        for turret in self.turrets:
            for bullet in turret.bullets:
                grid_x = int(30 - ((600 - bullet.x) / 20))
                grid_y = int(30 - ((600 - bullet.y) / 20))
                if (grid_x, grid_y) in wall_grid:
                    wall = wall_grid[(grid_x, grid_y)]
                    if circle_rectangle(wall.x, wall.y, 10,
                            bullet.x, bullet.y, bullet.width, bullet.height):
                        turret.bullets.remove(bullet)
                if (grid_x, grid_y) in human_cells:
                    if circle_rectangle(self.human.x, self.human.y, 10,
                            bullet.x, bullet.y, bullet.width, bullet.height):
                        self.on_loss()
                if (grid_x, grid_y) in robot_cells:
                    if circle_rectangle(self.robot.x, self.robot.y, 10,
                            bullet.x, bullet.y, bullet.width, bullet.height):
                        turret.bullets.remove(bullet)
                        bullet_hit.play()
                if grid_x > 29 or grid_x < 0 or\
                        grid_y > 29 or grid_y < 0:
                    turret.bullets.remove(bullet)

    def col_cells(self, sprite):
        grid_x = int(30 - ((600 - sprite.x) / 20))
        grid_y = int(30 - ((600 - sprite.y) / 20))
        return [(grid_x, grid_y), (grid_x - 1, grid_y), (grid_x - 1, grid_y + 1),
                (grid_x, grid_y + 1), (grid_x + 1, grid_y + 1),
                (grid_x + 1, grid_y), (grid_x + 1, grid_y - 1),
                (grid_x, grid_y - 1), (grid_x - 1, grid_y - 1)]

    def on_loss(self):
        self.window.set_scene(Play(self.level))

    def on_win(self):
        self.window.set_mouse_visible()
        self.window.set_scene(Win())

    def draw(self):
        self.background.draw()
        self.misc_batch.draw()
        self.robot.draw()
        self.human.draw()
        self.exit_batch.draw()
        self.wall_batch.draw()

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.set_mouse_visible()
            self.window.set_scene(scenes.Select(Play))
            return pyglet.event.EVENT_HANDLED
        if symbol == pyglet.window.key.SPACE:
            self.on_loss()
        if symbol == pyglet.window.key.A:
            self.human.say('stop')
            robot = self.robot
            if circle_rectangle(self.human.x, self.human.y, self.human.volume,
                        robot.x, robot.y, robot.width, robot.height):
                robot.stop()
            return
        if symbol == pyglet.window.key.S:
            self.human.say('flip')
            robot = self.robot
            if circle_rectangle(self.human.x, self.human.y, self.human.volume,
                        robot.x, robot.y, robot.width, robot.height):
                robot.flip()
            return
        if symbol == pyglet.window.key.D:
            self.human.say('turn')
            robot = self.robot
            if circle_rectangle(self.human.x, self.human.y, self.human.volume,
                        robot.x, robot.y, robot.width, robot.height):
                robot.turn()
            return
        if symbol == pyglet.window.key.F:
            self.human.say('go')
            robot = self.robot
            if circle_rectangle(self.human.x, self.human.y, self.human.volume,
                        robot.x, robot.y, robot.width, robot.height):
                robot.go()
            return
        self[symbol] = True
