import os
import random

import pyglet
import pygame

import scenes
from colors import *

# Resources
background = pyglet.resource.image('background.png')
frame = pyglet.image.SolidColorImagePattern((0, 0, 0, 30)).create_image(150, 150)


class Select(object):
    def __init__(self, exit_scene):
        self.exit_scene = exit_scene
        self.batch = pyglet.graphics.Batch()
        self.frame_batch = pyglet.graphics.Batch()
        self.background = pyglet.sprite.Sprite(background, 0, 0)
        self.page = 0
        self.thumbs = []
        self.frames = []
        self.maps = os.listdir(os.path.join('data', 'map'))
        self.maps.remove('thumb')
        self.maps.sort()
        self.coords = [[45, 425], [225, 425], [405, 425]
                      ,[45, 245], [225, 245], [405, 245]
                      ,[45, 65], [225, 65], [405, 65]]
        self.next = pyglet.text.Label('Next', x=430, y=23,
                        font_name="Another Typewriter", font_size=16, color=grey)
        self.prev = pyglet.text.Label('Prev', x=100, y=23,
                        font_name="Another Typewriter", font_size=16, color=grey)
        self.random = pyglet.text.Label('Random', x=225, y=20,
                        font_name="Another Typewriter", font_size=22, color=(171, 215, 95, 155))

        self.load_page()

    def load_page(self):
        self.thumbs = []
        self.frames = []
        for n, n2 in enumerate(range(self.page*8, self.page*8+9)):
            if n2 < len(self.maps):
                name = self.maps[n2]
                path = os.path.join('data', 'map', 'thumb', name+'.png')
                img = pyglet.image.load(path)
                self.thumbs.append([name, pyglet.sprite.Sprite(img,
                                                           self.coords[n][0],
                                                           self.coords[n][1],
                                                           batch=self.batch)])
                self.frames.append([name, pyglet.sprite.Sprite(frame,
                                                           self.coords[n][0]-10,
                                                           self.coords[n][1]+10,
                                                           batch=self.frame_batch)])
     
    def on_mouse_press(self, x, y, button, modifiers):
        for level, thumb in self.thumbs:
            if x >= thumb.x and x <=  thumb.x + thumb.width and\
                y >= thumb.y and y <= thumb.y + thumb.height:
                    pygame.mixer.Sound(os.path.join('data', 'music', 'thump.ogg')).play()
                    self.window.set_scene(self.exit_scene(level))

        if x >= self.next.x and x <=  self.next.x + self.next.content_width and\
            y >= self.next.y and y <= self.next.y + self.next.content_height:
                self.page += 1
                self.load_page()

        if x >= self.prev.x and x <=  self.prev.x + self.prev.content_width and\
            y >= self.prev.y and y <= self.prev.y + self.prev.content_height:
                self.page -= 1
                if self.page < 0:
                    self.window.set_scene(scenes.Menu())
                self.load_page()

        if x >= self.random.x and x <=  self.random.x + self.random.content_width and\
            y >= self.random.y and y <= self.random.y + self.random.content_height:
                pygame.mixer.Sound(os.path.join('data', 'music', 'thump.ogg')).play()
                self.window.set_scene(self.exit_scene(random.choice(self.maps)))

    def on_key_press(self, symbol, modifiers):
        if symbol == pyglet.window.key.ESCAPE:
            self.window.set_scene(scenes.Menu())
            return pyglet.event.EVENT_HANDLED

    def update(self, dt):
        pass

    def draw(self):
        self.background.draw()
        self.frame_batch.draw()
        self.batch.draw()
        self.next.draw()
        self.prev.draw()
        self.random.draw()
